package com.persagy.dmp.rwd.dic;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.ObjType;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.persagy.dmp.rwd.model.RwdeditRefTypeInfosModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;

import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 导入类型定义
 *
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Slf4j
public class ImportKnowledgeForEditV2 {

    private String basepath = "D:\\资料\\工作资料\\原型图\\数据中台\\数据字典数据";

    private String outPath = "D:/资料/工作资料/原型图/数据中台/out/ref-infos-" + DateUtils.format(new Date()) + ".json";

    private final static Integer CREATE_USER_ID=0;
    private final static Integer UPDATE_USER_ID=0;
    private final static ObjectMapper objectMapper = new ObjectMapper();

    @Test
    public void importClassData() throws Exception {

//        // 所有类型
        List<RwdeditRefTypeInfosModel> classes = new ArrayList<>();

        // 省市区编号
        initCityLocation(classes,"cityLocation","省市区编号");
        writeFuncSqlToFile(classes, "cityLocation");
        classes.clear();
    }

    public void initCityLocation(List<RwdeditRefTypeInfosModel> models, String refKey, String sheetName) throws Exception {
        String file = basepath + "/附表14-地域性信息表.xlsx";

        FileInputStream inputStream = new FileInputStream(file);
        Workbook workbook = new XSSFWorkbook(inputStream);
        Sheet sheet = workbook.getSheet(sheetName);

        // 行号从0开始, 从第2行开始读数据
        int startRowNum = 2;
        int lastRowNum = sheet.getLastRowNum();
        CacheModel cache = new CacheModel();
        for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
            Row row = sheet.getRow(rowIndex);
            if (row == null) {
                log.info("row[{}] is null", rowIndex);
                continue;
            }
            // 0专业, 3系统编码, 5系统名称, 6设备编码, 8设备名称
            String cityCode = (String) ExcelUtils.parseCell(row.getCell(2), "string");// 市级code
            String areaCode = (String) ExcelUtils.parseCell(row.getCell(4), "string");// 区级code
            String name = (String) ExcelUtils.parseCell(row.getCell(5), "string");// 名称
            Integer codeInt = (Integer)ExcelUtils.parseCell(row.getCell(6), "integer");// code
            if (null==name || null == codeInt){
                continue;
            }
            String code = codeInt.toString();
            Double longitude = (Double) ExcelUtils.parseCell(row.getCell(7), "double");// 精度
            Double latitude = (Double) ExcelUtils.parseCell(row.getCell(8), "double");// 维度
            Double altitude = (Double) ExcelUtils.parseCell(row.getCell(9), "double");// 海拔
            String climaticDivisionCode = (String) ExcelUtils.parseCell(row.getCell(10), "string");// 气候区
            String cityDevelopmentLevelCode = (String) ExcelUtils.parseCell(row.getCell(11), "string");// 发展水平
            RwdeditRefTypeInfosModel model = new RwdeditRefTypeInfosModel();
            model.setRefKey(refKey);
            model.setCode(code);
            model.setName(name);
            model.setParentCode(cache.equipCode);
            model.setCodePath(refKey+"-"+cache.systemCode+"-"+cache.equipCode+"-"+code);
            if (StringUtils.isNotBlank(climaticDivisionCode)){
                cache.climaticDivisionCode = climaticDivisionCode;
            }
            ObjectNode objectNode = objectMapper.createObjectNode()
                    .put("altitude", altitude)
                    .put("latitude", latitude)
                    .put("longitude", longitude)
                    .put("climaticDivisionCode", cache.climaticDivisionCode)
                    .put("cityDevelopmentLevelCode", cityDevelopmentLevelCode);
            objectNode.put("levelName","第三级");
            if ("00".equals(cityCode) && "00".equals(areaCode)){
                // 省
                cache.systemCode = code;
                cache.systemName = name;
                model.setParentCode("-9999");
                model.setCodePath(refKey+"-"+code);
                objectNode.put("levelName","第一级");
            }
            if (!"00".equals(cityCode) && "00".equals(areaCode)){
                // 市
                cache.equipCode = code;
                cache.equipName = name;
                model.setParentCode(cache.systemCode);
                model.setCodePath(refKey+"-"+cache.systemCode+"-"+code);
                objectNode.put("levelName","第二级");
            }
            model.setDataSource(objectNode);
            model.setCreateUser(CREATE_USER_ID);
            model.setUpdateUser(UPDATE_USER_ID);
            models.add(model);
        }
    }

    private void writeFuncSqlToFile(List<RwdeditRefTypeInfosModel> classes,String refKey) throws IOException {
        FileWriter writer = new FileWriter(outPath,true);
        List<Province> normal = new ArrayList<>();
        List<City> specialCities = new ArrayList<>();
        Map<String,List<RwdeditRefTypeInfosModel>> infoMap = new HashMap<>();
        for (RwdeditRefTypeInfosModel model : classes) {
            String codePath = model.getCodePath();
            List<RwdeditRefTypeInfosModel> subs = infoMap.getOrDefault(codePath.substring(0, codePath.lastIndexOf("-")), new ArrayList<>());
            subs.add(model);
            infoMap.put(codePath.substring(0, codePath.lastIndexOf("-")),subs);
        }
        List<RwdeditRefTypeInfosModel> models = classes.stream().sorted(Comparator.comparing(RwdeditRefTypeInfosModel::getCode)).collect(Collectors.toList());
        for (int i = 0; i < models.size(); i++) {
            RwdeditRefTypeInfosModel model = models.get(i);
            if (model.getName().contains("香港")
                    || model.getName().contains("澳门")
                    || model.getName().contains("台湾")){
                continue;
            }
            if (model.getName().contains("北京市")
                    || model.getName().contains("天津市")
                    || model.getName().contains("重庆市")
                    || model.getName().contains("上海市")){
                City city = new City();
                city.setCityCode(model.getCode());
                city.setCityName(model.getName());
                List<CityInfo> list = new ArrayList<>();
                list.add(CityInfo.builder()
                        .code(model.getCode())
                        .name(model.getName())
                        .altitude(model.getDataSource().get("altitude")!=null?model.getDataSource().get("altitude").textValue():"")
                        .latitude(model.getDataSource().get("latitude")!=null?model.getDataSource().get("latitude").textValue():"")
                        .longitude(model.getDataSource().get("longitude")!=null?model.getDataSource().get("longitude").textValue():"")
                        .build());
                list.add(CityInfo.builder()
                        .code(model.getCode().substring(0,4)+"99")
                        .name(model.getName()+"市本级")
                        .build());
                city.setCityInfo(list);
                city.setClimate(model.getDataSource().get("climaticDivisionCode")!=null?model.getDataSource().get("climaticDivisionCode").textValue():"");
                city.setDevelopLevel(model.getDataSource().get("cityDevelopmentLevelCode")!=null?model.getDataSource().get("cityDevelopmentLevelCode").textValue():"");
                city.setDistrict(transferToDistrict(infoMap.getOrDefault(model.getCodePath()+"-"+model.getCode().substring(0,2)+"0100",new ArrayList<>())));
                specialCities.add(city);
                continue;
            }
            if (!"第一级".equals(model.getDataSource().get("levelName").toString())){
                continue;
            }
            Province province = Province.builder()
                    .provinceCode(model.getCode())
                    .provinceName(model.getName())
                    .provinceInfo(Arrays.asList(ProvinceInfo.builder().code(model.getCode()).name(model.getName()).build(),
                            ProvinceInfo.builder().code(model.getCode().substring(0, 2) + "9900").name(model.getName() + "省本级").build()))
                    .build();
            province.setCities(transferToCity(infoMap.getOrDefault(model.getCodePath(),new ArrayList<>()),infoMap));
            normal.add(province);
        }
        // 处理省
        List<RwdeditRefTypeInfosModel> provines = infoMap.getOrDefault(refKey, new ArrayList<>());
        for (RwdeditRefTypeInfosModel model : provines) {
            if (model.getName().contains("香港")
                    || model.getName().contains("澳门")
                    || model.getName().contains("台湾")){
                continue;
            }
            if (model.getName().contains("北京市")
                    || model.getName().contains("天津市")
                    || model.getName().contains("重庆市")
                    || model.getName().contains("上海市")){
                continue;
            }
            Province province = Province.builder()
                    .provinceCode(model.getCode())
                    .provinceName(model.getName())
                    .provinceInfo(Arrays.asList(ProvinceInfo.builder().code(model.getCode()).name(model.getName()).build(),
                            ProvinceInfo.builder().code(model.getCode().substring(0, 2) + "9900").name(model.getName() + "省本级").build()))
                    .build();
            province.setCities(transferToCity(infoMap.getOrDefault(model.getCodePath(),new ArrayList<>()),infoMap));
            normal.add(province);
        }
        Map<String, JSONArray> criteria = new HashMap<>();
        criteria.put("normal", JSONUtil.parseArray(JSONUtil.toJsonStr(normal)));
        criteria.put("specialDistrict",JSONUtil.parseArray("[{\"name\":\"香港\",\"code\":\"701000\",\"longitude\":\"114.1\",\"latitude\":\"22.2\",\"altitude\":\"32\",\"climate\":\"40\",\"developLevel\":\"1610\"},{\"name\":\"澳门\",\"code\":\"702000\",\"longitude\":\"113.5\",\"latitude\":\"22.2\",\"altitude\":\"57\",\"climate\":\"40\",\"developLevel\":\"1610\"},{\"name\":\"台湾\",\"code\":\"703000\",\"longitude\":\"121.3\",\"latitude\":\"25\",\"altitude\":\"9\",\"climate\":\"40\",\"developLevel\":\"1620\"}]"));
        criteria.put("specialCities",JSONUtil.parseArray(JSONUtil.toJsonStr(specialCities)));
        writer.write(criteria.toString());
        writer.flush();
        writer.close();
    }

    private List<City> transferToCity(List<RwdeditRefTypeInfosModel> models,Map<String,List<RwdeditRefTypeInfosModel>> infoMap) {
        List<City> result = new ArrayList<>();
        if (CollUtil.isEmpty(models)){
            return new ArrayList<>();
        }
        for (RwdeditRefTypeInfosModel model : models) {
            City city = new City();
            city.setCityCode(model.getCode());
            city.setCityName(model.getName());
            List<CityInfo> list = new ArrayList<>();
            list.add(CityInfo.builder()
                    .code(model.getCode())
                    .name(model.getName())
                    .altitude(model.getDataSource().get("altitude")!=null?model.getDataSource().get("altitude").textValue():"")
                    .latitude(model.getDataSource().get("latitude")!=null?model.getDataSource().get("latitude").textValue():"")
                    .longitude(model.getDataSource().get("longitude")!=null?model.getDataSource().get("longitude").textValue():"")
                    .build());
            list.add(CityInfo.builder()
                    .code(model.getCode().substring(0,4)+"99")
                    .name(model.getName()+"市本级")
                    .build());
            city.setCityInfo(list);
            city.setClimate(model.getDataSource().get("climaticDivisionCode")!=null?model.getDataSource().get("climaticDivisionCode").textValue():"");
            city.setDevelopLevel(model.getDataSource().get("cityDevelopmentLevelCode")!=null?model.getDataSource().get("cityDevelopmentLevelCode").textValue():"");
            city.setDistrict(transferToDistrict(infoMap.getOrDefault(model.getCodePath(),new ArrayList<>())));
            result.add(city);
        }
        return result;
    }

    private List<District> transferToDistrict(List<RwdeditRefTypeInfosModel> list) {
        List<District> result = new ArrayList<>();
        if (CollUtil.isEmpty(list)){
            return new ArrayList<>();
        }
        for (RwdeditRefTypeInfosModel model : list) {
            District district = District.builder()
                    .code(model.getCode())
                    .districtName(model.getName())
                    .districtCode(model.getCode().substring(model.getCode().length() - 2, model.getCode().length()))
                    .altitude(model.getDataSource().get("altitude") != null ? model.getDataSource().get("altitude").textValue() : "")
                    .latitude(model.getDataSource().get("latitude") != null ? model.getDataSource().get("latitude").textValue() : "")
                    .longitude(model.getDataSource().get("longitude") != null ? model.getDataSource().get("longitude").textValue() : "")
                    .developLevel(model.getDataSource().get("cityDevelopmentLevelCode") != null ? model.getDataSource().get("cityDevelopmentLevelCode").textValue() : "")
                    .build();
            result.add(district);
        }
        return result;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class Province{
        private String provinceCode;
        private String provinceName;
        private List<ProvinceInfo> provinceInfo;
        private List<City> cities;
    }
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class ProvinceInfo{
        private String name;
        private String code;
    }
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class City{
        private String cityCode;
        private String cityName;
        private String climate;
        private String developLevel;
        private List<CityInfo> cityInfo;
        private List<District> district;
    }
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class CityInfo{
        private String name;
        private String code;
        private String longitude;
        private String latitude;
        private String altitude;
    }
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class District{
        private String districtCode;
        private String districtName;
        private String code;
        private String longitude;
        private String latitude;
        private String altitude;
        private String developLevel;
    }
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class DistrictInfo{
        private String name;
        private String code;
        private String longitude;
        private String latitude;
        private String altitude;
        private String climate;
        private String developLevel;
    }

    public static final class CacheModel {
        String majorCode;
        String majorEnglishName;
        String majorName;
        String systemCode;
        String systemEnglishName;
        String systemName;
        String equipCode;
        String equipEnglishName;
        String equipName;
        String componentCode;
        String componentName;
        String componentEnglishName;
        String climaticDivisionCode;
        String extendName;
        String firstLevelCode;
        String firstLevelName;
        String secondLevelCode;
        String secondLevelName;
        String thirdLevelCode;
        String thirdLevelName;
        String fourthLevelCode;
        String fourthLevelName;
        String fifthLevelCode;
        String fifthLevelName;
        String sixthLevelCode;
        String sixthLevelName;
        String seventhLevelCode;
        String seventhLevelName;
    }

}
