package com.persagy.dmp.rwd.edit.service;

import com.persagy.dmp.common.excel.ExcelUtils;
import com.persagy.dmp.common.excel.SheetWriteInfo;
import com.persagy.dmp.rwd.edit.entity.ClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.FuncidDefChangeRecord;
import com.persagy.dmp.rwd.edit.model.DownloadModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Slf4j
@Service
public class DownloadService {

    public DownloadModel download(String type) {
        if ("changeDataTemplate".equals(type)) {
            return downloadChangeDataTemplate();
        }
        return new DownloadModel();
    }

    private DownloadModel downloadChangeDataTemplate() {
        DownloadModel result = new DownloadModel();
        result.setType("workbook");
        result.setName("批量上传变更记录.xlsx");

        List<SheetWriteInfo> list = new ArrayList<>(2);
        list.add(calendarReportPart1());
        list.add(calendarReportPart2());
        list.add(calendarReportPart3());
        Workbook workbook = ExcelUtils.createExcel(list);
        result.setWorkbook(workbook);
        return result;
    }

    public SheetWriteInfo calendarReportPart1() {
        SheetWriteInfo<String> info = new SheetWriteInfo<>();
        List<String> list = new ArrayList<>();
        list.add("1.不可合并单元格");
        list.add("2.sheet1：类型变更");
        list.add("3.sheet2：信息点变更");
        list.add("4.sheet2-'字典选择'列：枚举类型用空格隔开；区间‘()’表示不包含‘[]’表示包含，例如：（10，20] [50，70]，多个区间使用空格隔开");
        list.add("5.sheet2-'信息点分类'列和'信息点类型'列：参考：http://39.102.54.110:9003/rwd/def_funcid.html 文档");
        list.add("6.文件格式为：xlsx");
        info.setData(list);
        info.setSheetName("文件格式要求");
        info.addColumn("文件格式要求", "type", "string", 120, HorizontalAlignment.LEFT);
        info.setDataHandler((data, prop) -> {
            return data;
        });
        return info;
    }

    public SheetWriteInfo calendarReportPart2() {
        SheetWriteInfo<ClassDefChangeRecord> info = new SheetWriteInfo<>();
        info.setData(Arrays.asList(new ClassDefChangeRecord()));
        info.setSheetName("类型变更");
        info.addColumn("专业编码", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("专业英文名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("专业名称", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("系统类编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("系统别编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("系统类英文名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("系统类名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("设备类编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("设备类别编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("设备类英文名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("设备类名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("部件类编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("部件别编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("部件类英文名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("部件类名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.setDataHandler((data, prop) -> {
            return null;
        });
        return info;
    }

    public SheetWriteInfo calendarReportPart3() {
        SheetWriteInfo<FuncidDefChangeRecord> info = new SheetWriteInfo<>();
        info.setData(Arrays.asList(new FuncidDefChangeRecord()));
        info.setSheetName("信息点变更");
        info.addColumn("类型编码", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("类型名称", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("一级标签", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("二级标签", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("信息点编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("信息点别编码", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("信息点名称", "name", "string", 20, HorizontalAlignment.CENTER);
        info.addColumn("信息点分类", "category", "array", 20, HorizontalAlignment.CENTER);
        info.addColumn("优先级", "priority", "array", 10, HorizontalAlignment.CENTER);
        info.addColumn("单位", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("信息点类型", "dataType", "array", 20, HorizontalAlignment.CENTER);
        info.addColumn("是否复数", "isMultiple", "array", 10, HorizontalAlignment.CENTER);
        info.addColumn("是否区间", "isRegion", "array", 10, HorizontalAlignment.GENERAL);
        info.addColumn("字典选择", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("数据格式", "name", "string", 10, HorizontalAlignment.CENTER);
        info.addColumn("备注", "name", "string", 10, HorizontalAlignment.CENTER);
        info.setDataHandler((data, prop) -> {
            switch (prop) {
                case ("dataType"): {
                    return new String[]{"INTEGER", "DOUBLE", "BOOLEAN", "STRING", "DATETIME", "ENUM", "MENUM", "REFENMUM"};
                }
                case ("category"): {
                    return new String[]{"STATIC", "PULSE", "SEQUENTIAL", "GRADATION"};
                }
                case ("priority"): {
                    return new String[]{"A2", "D1", "M", "R", "S"};
                }
                case ("isRegion"):
                case ("isMultiple"): {
                    return new String[]{"1", "0"};
                }
                default: {
                    return null;
                }
            }
        });
        return info;
    }
}

