package com.persagy.dmp.rwd.edit.service;

import cn.hutool.core.collection.CollUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.config.web.UserUtils;
import com.persagy.dmp.rwd.edit.entity.QRwdeditRefTypeInfos;
import com.persagy.dmp.rwd.edit.entity.RwdeditRefType;
import com.persagy.dmp.rwd.edit.entity.RwdeditRefTypeInfos;
import com.persagy.dmp.rwd.edit.model.RwdeditRefTypeModel;
import com.persagy.dmp.rwd.edit.repository.RwdeditRefTypeInfosRepository;
import com.persagy.dmp.rwd.model.RwdeditRefTypeInfosModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class RwdeditRefTypeInfosService {

    @Autowired
    private CriteriaUtils criteriaUtils;

    @Autowired
    RwdeditRefTypeInfosRepository repository;

    private List<BooleanExpression> parse(ObjectNode object) {
        List<BooleanExpression> list = new LinkedList<>();
//        QRwdeditRefTypeInfos qt = QRwdeditRefTypeInfos.rwdeditRefTypeInfos;
//        list.addAll(CriteriaUtils.parse(qt.refKey, object.get("refKey")));
//        list.addAll(CriteriaUtils.parse(qt.code, object.get("code")));
//        list.addAll(CriteriaUtils.parse(qt.createUser, object.get("createUser")));
//        list.addAll(CriteriaUtils.parse(qt.updateUser, object.get("updateUser")));
        return list;
    }

    public PagedResponse<RwdeditRefTypeInfosModel> query(JacksonCriteria criteria) {
        // 1.根据条件查询数据
        PagedResponse<RwdeditRefTypeInfos> destRefTypeInfos = criteriaUtils.query(QRwdeditRefTypeInfos.rwdeditRefTypeInfos, this::parse, criteria);
        PagedResponse<RwdeditRefTypeInfosModel> result = new PagedResponse<>();
        List<RwdeditRefTypeInfos> dataList = destRefTypeInfos.getData();
        if (CollUtil.isNotEmpty(dataList)) {
            // 2.根据父级code查询数据
            ArrayNode parentCodes = JacksonMapper.nonDefaultMapper.createArrayNode();
            Set<String> filterIds = new HashSet<>();
            dataList.forEach(entity->{
                if (!filterIds.contains(entity.getCodePath())){
                    parentCodes.add(entity.getCodePath().substring(0,entity.getCodePath().lastIndexOf("-")));
                    filterIds.add(entity.getCodePath());
                }
            });
            JacksonCriteria allCriteria = JacksonCriteria.newInstance();
            ObjectNode objectNode = JacksonMapper.nonDefaultMapper.createObjectNode();
            ObjectNode parentCodeNode = JacksonMapper.nonDefaultMapper.createObjectNode();
            parentCodeNode.put("$in",parentCodes);
            objectNode.put("codePath", parentCodeNode);
            allCriteria.setCriteria(objectNode);
            allCriteria.setWithColumns(new HashSet<>(Arrays.asList("code","name","codePath")));
            allCriteria.setWithoutCount(true);
            PagedResponse<RwdeditRefTypeInfos> allRefTypeInfos = criteriaUtils.query(QRwdeditRefTypeInfos.rwdeditRefTypeInfos, this::parse, allCriteria);
            List<RwdeditRefTypeInfos> allRefTypeInfosData = allRefTypeInfos.getData();
            Map<String, String> nameMap = allRefTypeInfosData.stream()
                    .collect(Collectors
                            .toMap(RwdeditRefTypeInfos::getCodePath, RwdeditRefTypeInfos::getName, (k1, k2) -> k1));
            List<RwdeditRefTypeInfosModel> collect = dataList.stream().map(entity -> {
                RwdeditRefTypeInfosModel model = entity.toModel();
                ObjectNode dataSource = entity.getDataSource();
                if (null==dataSource){
                    dataSource = JacksonCriteria.newInstance().getCriteria();
                }
                String parentName = nameMap.getOrDefault(entity.getCodePath().substring(0, entity.getCodePath().lastIndexOf("-")), "");
                dataSource.put("code",entity.getCode());
                dataSource.put("name",entity.getName());
                dataSource.put("parentCode",entity.getParentCode());
                dataSource.put("parentName",parentName);
                model.setDataSource(dataSource);
                model.setParentName(parentName);
                return model;
            }).collect(Collectors.toList());
            result.setData(collect);
        }
        result.setCount(destRefTypeInfos.getCount());
        return result;
    }

    @Transactional
    public MapResponse create(RwdeditRefTypeInfos refTypeInfos) {
        MapResponse response = new MapResponse();
        String refKey = refTypeInfos.getRefKey();
        String code = refTypeInfos.getCode();
        if (refKey == null) {
            response.setFail("refKey is required");
            return response;
        }
        if (code == null) {
            response.setFail("code is required");
            return response;
        }
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("refKey", refKey);
        criteria.add("code", code);
        Long count = query(criteria).getCount();
        if (count != 0L) {
            response.setFail("refKey and code is exists");
            return response;
        }
        refTypeInfos.setCreateUser(UserUtils.currentUserId());
        repository.save(refTypeInfos);
        response.add("refKey", refKey);
        response.add("code", code);
        return response;
    }

    @Transactional
    public MapResponse update(RwdeditRefTypeInfos refTypeInfos) {
        MapResponse response = new MapResponse();
        String refKey = refTypeInfos.getRefKey();
        String code = refTypeInfos.getCode();
        if (refKey == null) {
            response.setFail("refKey is required");
            return response;
        }
        if (code == null) {
            response.setFail("code is required");
            return response;
        }
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("refKey", refKey);
        criteria.add("code", code);
        Long count = query(criteria).getCount();
        if (count == 0L) {
            response.setFail("refKey and code not exists");
            return response;
        }
        RwdeditRefTypeInfosModel model = query(criteria).getData().get(0);
        RwdeditRefTypeInfos entity = RwdeditRefTypeInfos.fromModel(model);
        entity.setUpdateUser(UserUtils.currentUserId());
        if (refTypeInfos.getName() != null) {
            entity.setName(refTypeInfos.getName());
        }

        repository.save(entity);
        return response;
    }

    @Transactional
    public MapResponse delete(RwdeditRefTypeInfos refTypeInfos) {
        MapResponse response = new MapResponse();
        String refKey = refTypeInfos.getRefKey();
        String code = refTypeInfos.getCode();
        if (refKey == null) {
            response.setFail("refKey is required");
            return response;
        }
        if (code == null) {
            response.setFail("code is required");
            return response;
        }
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("refKey", refKey);
        criteria.add("code", code);
        PagedResponse<RwdeditRefTypeInfosModel> query = query(criteria);
        Long count = query.getCount();
        if (count == 0L) {
            response.setFail("refKey and code not exists");
            return response;
        }
        repository.delete(RwdeditRefTypeInfos.fromModel(query.getData().get(0)));
        return response;
    }
}

