package com.persagy.dmp.rwd.datautils;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.core.util.IdUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.dic.DataTypeMapping;
import com.persagy.dmp.rwd.dic.ExcelUtils;
import com.persagy.dmp.rwd.dic.FuncidDefModel;
import com.persagy.dmp.rwd.dic.FuncidUtils;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.*;
import java.text.DateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Slf4j
@SpringBootTest
@RunWith(SpringRunner.class)
public class ImportFuncidForRwdEdit {

    private String basepath = "D:\\资料\\工作资料\\原型图\\数据中台\\数据字典数据";

    private String outPath = "D:/资料/工作资料/原型图/数据中台/out/funcid-" + DateUtils.format(new Date()) + ".dml.sql";

    private final static String VERSION = "3.1.11";

    private AtomicInteger systemCommonSeq = new AtomicInteger(0);
    private AtomicInteger equipCommonSeq = new AtomicInteger(0);
    private AtomicInteger componentCommonSeq = new AtomicInteger(0);

    private final static Snowflake SNOWFLAKE = IdUtil.createSnowflake(12,15);

    @Autowired
    private HttpUtils httpUtils;

    @Value("${persagy.dmp.rwdedit.server}")
    private String rwdeditServer;

    @Before
    public void init() {
        FuncidUtils.httpUtils = httpUtils;
        FuncidUtils.rwdeditServer = rwdeditServer;
    }

    @Test
    public void importFuncidDef() throws IOException {
        List<FuncidDefModel> funcids = new LinkedList<>();
//        importBase(funcids);
        importSpace(funcids);
//        importSystem(funcids);
//        importEquipment(funcids);
//        importComponent(funcids);
        //importVirtual(funcids);
//        importTool(funcids);
//        importMaterial(funcids);
    }

    private void writeFuncSqlToFile(List<FuncidDefModel> funcids,String classCode) throws IOException {
        FileWriter writer = new FileWriter(outPath,true);
        writer.append("delete from rwd_def_funcid where class_code='"+classCode+"';");
        writer.append("\n");
        writer.append("commit;");
        writer.append("\n");
        for (FuncidDefModel item : funcids) {
            writer.append(toSql(item));
            writer.append("\n");
        }
        writer.append("commit;");
        writer.append("\n");
        writer.flush();
        writer.close();
    }


    private void importBase(List<FuncidDefModel> funcids) throws IOException {
        funcids.addAll(readSheet("system", "/附表6-机电精装类对象通用属性点表.xlsx", "系统类"));
        funcids.addAll(readSheet("equipment", "/附表6-机电精装类对象通用属性点表.xlsx", "设备类"));
        funcids.addAll(readSheet("component", "/附表6-机电精装类对象通用属性点表.xlsx", "部件类"));

    }

    private String toSql(FuncidDefModel model) {
        StringBuilder buff = new StringBuilder("");
        buff.append("insert into rwd_def_funcid ");
        buff.append("(id,code,orig_code,name,alias_code,alias_name,class_code,type,group_code,project_id,category,");
        buff.append("first_tag,second_tag,priority,input_mode,unit,data_type,orig_data_type,is_multiple,is_region,formater,data_source,orig_data_source,note,sub_flag,weak_point,");
        buff.append("is_used,version,last_update_time,share_type,order_seq) \n");
        buff.append("values (");
        buff.append("'").append(SNOWFLAKE.nextIdStr()).append("',"); // id
        buff.append("'").append(model.getCode()).append("',"); // code
        buff.append("'").append(model.getOrigCode()).append("',"); // orig_code
        buff.append("'").append(model.getName()).append("',"); // name
        buff.append("'").append(model.getAliasCode()).append("',"); // alias_code
        buff.append("'").append(model.getAliasName()).append("',"); // alias_name
        buff.append("'").append(model.getClassCode()).append("',"); // class_code
        buff.append("'").append(model.getType()).append("',"); // type
        buff.append("'").append(model.getGroupCode()).append("',"); // group_code
        buff.append("'").append(model.getProjectId()).append("',"); // project_id
        buff.append("'").append(model.getCategory().name()).append("',"); // category
        if (model.getFirstTag() != null) {
            buff.append("'").append(model.getFirstTag()).append("',"); // first_tag
        } else {
            buff.append("null,");
        }
        if (model.getSecondTag() != null) {
            buff.append("'").append(model.getSecondTag()).append("',"); // second_tag
        } else {
            buff.append("null,");
        }
        if (model.getPriority() != null) {
            buff.append("'").append(model.getPriority()).append("',"); // priority
        } else {
            buff.append("null,");
        }
        if (model.getInputMode() != null) {
            buff.append("'").append(model.getInputMode()).append("',"); // input_mode
        } else {
            buff.append("null,");
        }
        if (model.getUnit() != null) {
            buff.append("'").append(model.getUnit()).append("',"); // unit
        } else {
            buff.append("null,");
        }

        buff.append("'").append(model.getDataType().name()).append("',"); // data_type
        buff.append("'").append(model.getOrigDataType()).append("',"); // orig_data_type
        buff.append(model.getIsMultiple() ? 1 : 0).append(","); // is_multiple
        buff.append(model.getIsRegion() ? 1 : 0).append(","); // is_region
        if (model.getFormater() != null) {
            buff.append("'").append(model.getFormater()).append("',"); // formater
        } else {
            buff.append("null,");
        }
        if (model.getDataSource() != null) {
            buff.append("'").append(model.getDataSource().toString()).append("',"); // data_source
        } else {
            buff.append("null,");
        }
        if (model.getOrigDataSource() != null) { // orig_data_source
            buff.append("'").append(model.getOrigDataSource()).append("',"); // data_source
        } else {
            buff.append("null,");
        }
        if (model.getNote() != null) {
            buff.append("'").append(model.getNote()).append("',"); // note}
        } else {
            buff.append("null,");
        }
        buff.append(model.getSubFlag() ? 1 : 0).append(","); // sub_flag
        buff.append(model.getWeakPoint() ? 1 : 0).append(","); // weak_point
        buff.append(1).append(","); // is_used
        buff.append("'").append(VERSION).append("',"); // version
        buff.append("'").append(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(LocalDateTime.now())).append("',"); // last_update_time
        if (null!=model.getShareType()){// share_type
            buff.append("'").append(model.getShareType()).append("',");
        }else {
            buff.append("null,");
        }
        if (null!=model.getOrderSeq()){// order_seq
            buff.append(model.getOrderSeq()).append(",");
        }else {
            buff.append("null,");
        }
        buff.deleteCharAt(buff.length() - 1);
        buff.append(");");
        return buff.toString();
    }

    private FuncidDefModel toFuncidDefModel(CacheModel model) {
        FuncidDefModel funcid = new FuncidDefModel();
        funcid.setGroupCode("0");
        funcid.setProjectId("0");
        funcid.setType("common");
        funcid.setShareType(model.shareType);
        funcid.setOrderSeq(model.orderSeq);
        funcid.setClassCode(model.classCode);
        funcid.setFirstTag(model.firstTag != null ? model.firstTag.replace("\n", "") : null);
        funcid.setSecondTag(model.secondTag != null ? model.secondTag.replace("\n", "") : null);
        funcid.setCode(model.origCode.substring(0, 1).toLowerCase() + model.origCode.substring(1));
        funcid.setOrigCode(model.origCode);
        funcid.setAliasCode(model.origCode);
        if (model.name != null) {
            funcid.setName(model.name.replace("\n", ""));
            funcid.setAliasName(model.name.replace("\n", ""));
        } else {
            log.error("funcid name is null : {}", JacksonMapper.toSimpleJson(funcid));
        }
        funcid.setUnit(model.unit);
        funcid.setPriority(model.priority);
        funcid.setInputMode(model.inputMode);
        funcid.setOrigDataType(model.origDataType);
        funcid.setSubFlag(false);
        funcid.setWeakPoint(false);
        FuncidDefModel dataType = DataTypeMapping.getDataType(model.origDataType);
        if (dataType != null) {
            funcid.setCategory(dataType.getCategory());
            funcid.setDataType(dataType.getDataType());
            funcid.setIsMultiple(dataType.getIsMultiple());
            funcid.setIsRegion(dataType.getIsRegion());
        } else {
            return null;
        }
        funcid.setSubFlag(false);
        if (model.remark != null) {
            funcid.setNote(model.remark.replace("\n", ""));
            if (model.remark != null) {
                if (model.remark.contains("YYYYMMDD-YYYYMMDD")) {
                    funcid.setFormater("YYYYMMDD-YYYYMMDD");
                }else if (model.remark.contains("YYYYMMDD")) {
                    funcid.setFormater("YYYYMMDD");
                }else if (model.remark.contains("YYYY")) {
                    funcid.setFormater("YYYY");
                }
            }
        }

        if (model.origDataSource != null) {
            funcid.setOrigDataSource(model.origDataSource.replace("\n", "nn")); // 去掉换行符
        }
        if (funcid.getDataType() == FuncidDataType.ENUM || funcid.getDataType() == FuncidDataType.MENUM || funcid.getDataType() == FuncidDataType.BOOLEAN) {
            funcid.setDataSource(FuncidUtils.transEnumDataSource(model.origDataSource)); // 解析datasource
        }
        return funcid;
    }

    private void importSpace(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("project", "/附表5-建筑空间类对象属性点表/附表5.1-项目.xlsx", "项目","2");
        analysisSheetDataAndWriteToFile("building", "/附表5-建筑空间类对象属性点表/附表5.2-建筑.xlsx", "建筑","2");
        analysisSheetDataAndWriteToFile("floor", "/附表5-建筑空间类对象属性点表/附表5.3-楼层.xlsx", "楼层","2");
        analysisSheetDataAndWriteToFile("shaft", "/附表5-建筑空间类对象属性点表/附表5.4-竖井.xlsx", "竖井","2");
        analysisSheetDataAndWriteToFile("GeneralZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "CMZ物业分区","2");
        analysisSheetDataAndWriteToFile("PowerSupplyZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "PSZ供电分区","2");
        analysisSheetDataAndWriteToFile("LightingZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "LTZ照明分区","2");
        analysisSheetDataAndWriteToFile("NetworkZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "NWZ网络分区","2");
        analysisSheetDataAndWriteToFile("AirConditioningZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "ACZ空调分区","2");
        analysisSheetDataAndWriteToFile("HeatingZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "HTZ采暖分区","2");
        analysisSheetDataAndWriteToFile("CleanZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "CLZ洁净分区","2");
        analysisSheetDataAndWriteToFile("DomesticWaterSupplyZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "DWZ生活给水分区","2");
        analysisSheetDataAndWriteToFile("FireZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "FRZ防火分区","2");
        analysisSheetDataAndWriteToFile("SecurityZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "SCZ安防分区","2");
        analysisSheetDataAndWriteToFile("TenantZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "TNZ租户分区","2");
        analysisSheetDataAndWriteToFile("FunctionZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "FCZ功能分区","2");
        analysisSheetDataAndWriteToFile("PassengerFlowZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "PFZ客流分区","2");
        analysisSheetDataAndWriteToFile("EvacuateZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "EVZ疏散分区","2");
        analysisSheetDataAndWriteToFile("EnvironmentZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "ENZ环境分区","2");
        analysisSheetDataAndWriteToFile("OtherZone", "/附表5-建筑空间类对象属性点表/附表5.5-空间.xlsx", "OTZ其他分区","2");
    }

    private void analysisSheetDataAndWriteToFile(String classCode, String fileName, String sheetName,String shareType) throws IOException {
        // 1.解析Sheet数据
        List<FuncidDefModel> funcids = new ArrayList<>(readSheet(classCode, fileName, sheetName, shareType));
        // 2.写入文件
        writeFuncSqlToFile(funcids,classCode);
    }

    private void analysisSheetDataAndWriteToFile(String classCode, String fileName, String sheetName,String shareType,AtomicInteger commonSeq) throws IOException {
        // 1.解析Sheet数据
        List<FuncidDefModel> funcids = new ArrayList<>(readSheet(classCode, fileName, sheetName, shareType,commonSeq));
        // 2.写入文件
        writeFuncSqlToFile(funcids,classCode);
    }

    private void importSystem(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("system", "/附表6-机电精装类对象通用属性点表.xlsx", "系统类","1");
        AtomicInteger systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETD", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "TD变配电系统 ","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDG", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "DG柴发机房系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEBP", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "BP备用电源系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEEP", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "EP机房动力系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEOP", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "OP室外动力系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEEL", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "EL电梯系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELT", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "LT照明系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETP", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "TP租户配电系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELP", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "LP雷电防护系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SESD", "/附表7-系统类对象分类属性点表/附表7.1-强电专业系统.xlsx", "SD强电人防系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBA", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "BA建筑设备控制系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETV", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "TV电视信号系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMI", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "MI多媒体信息发布系统  ","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEWT", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "WT无线对讲系统   ","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGM", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "GM车库管理系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELN", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "LN本地网络系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMA", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "MA移动通信信号放大系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETM", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "TM租户管理系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEPM", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "PM客流管理系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEEM", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "EM能源管理系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMM", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "MM气象监测系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WECM", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "CM收银管理系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOF", "/附表7-系统类对象分类属性点表/附表7.2-弱电专业系统.xlsx", "OF办公系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCC", "/附表7-系统类对象分类属性点表/附表7.3-空调专业系统.xlsx", "CC中央供冷系统 ","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCH", "/附表7-系统类对象分类属性点表/附表7.3-空调专业系统.xlsx", "CH中央供热系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACAT", "/附表7-系统类对象分类属性点表/附表7.3-空调专业系统.xlsx", "AT空调末端系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVT", "/附表7-系统类对象分类属性点表/附表7.3-空调专业系统.xlsx", "VT通风系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACKL", "/附表7-系统类对象分类属性点表/附表7.3-空调专业系统.xlsx", "KL厨房排油烟系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACAD", "/附表7-系统类对象分类属性点表/附表7.3-空调专业系统.xlsx", "AD空调人防系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDW", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "DW生活给水系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDH", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "DH生活热水系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRW", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "RW中水系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDD", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "DD直饮水系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSST", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "ST污水处理系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSI", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "SI喷灌系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWS", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "WS水景系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWP", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "WP泳池系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSFG", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "FG燃气系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWH", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "WH给排水电伴热系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWD", "/附表7-系统类对象分类属性点表/附表7.4-给排水专业系统.xlsx", "WD给排水人防系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFA", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "FA火灾报警系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGA", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "GA燃气报警系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEA", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "EA漏电火灾报警系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBM", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "BM紧急广播及背景音乐系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFS", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "FS消防给水系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGE", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "GE气体灭火系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFE", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "FE灭火器系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSC", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "SC防排烟系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFR", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "FR防火卷帘门系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEE", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "EE应急照明及疏散指示系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFKE", "/附表7-系统类对象分类属性点表/附表7.5-消防专业系统.xlsx", "KE厨房自动灭火系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVS", "/附表7-系统类对象分类属性点表/附表7.6-安防专业系统.xlsx", "VS视频监控系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPNP", "/附表7-系统类对象分类属性点表/附表7.6-安防专业系统.xlsx", "NP巡更系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEG", "/附表7-系统类对象分类属性点表/附表7.6-安防专业系统.xlsx", "EG门禁系统","2",systemCommonSeq);
        systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIA", "/附表7-系统类对象分类属性点表/附表7.6-安防专业系统.xlsx", "IA入侵报警系统","2",systemCommonSeq);
    }

    private void importEquipment(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("equipment", "/附表6-机电精装类对象通用属性点表.xlsx", "设备类","1");
        AtomicInteger equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDTF", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDTF变压器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDHS", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDHS高压开关柜","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDLS", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDLS低压开关柜","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDDS", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDDS直流屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDIV", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDIV逆变器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDBP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDBP变配电母线插接箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.1-变配电系统设备.xlsx", "TDCU变配电控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SESDCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.10-强电人防系统设备.xlsx", "SDCU强电人防控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDGDG", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.2-柴发机房系统设备.xlsx", "DGDG柴油发电机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDGDT", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.2-柴发机房系统设备.xlsx", "DGDT柴发储油箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDGPP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.2-柴发机房系统设备.xlsx", "DGPP柴发输油泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDGJH", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.2-柴发机房系统设备.xlsx", "DGJH柴发水套加热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDGEP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.2-柴发机房系统设备.xlsx", "DGEP柴发尾气净化装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEDGCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.2-柴发机房系统设备.xlsx", "DGCU柴发控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEBPBP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.3-备用电源系统设备.xlsx", "BPBP备用电源","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEBPSP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.3-备用电源系统设备.xlsx", "BPSP太阳能电池板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEBPWG", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.3-备用电源系统设备.xlsx", "BPWG风力发电装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEBPHT", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.3-备用电源系统设备.xlsx", "BPHT备用电源主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEBPCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.3-备用电源系统设备.xlsx", "BPCU备用电源控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEEPCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.4-机房动力系统设备.xlsx", "EPCU机房动力控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEOPCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.5-室外动力系统设备.xlsx", "OPCU室外动力控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEELEL", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.6-电梯系统设备.xlsx", "ELEL直梯","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEELES", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.6-电梯系统设备.xlsx", "ELES扶梯","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEELCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.6-电梯系统设备.xlsx", "ELCU电梯控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SEELET", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.6-电梯系统设备.xlsx", "ELET电梯曳引机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELTLT", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.7-照明系统设备.xlsx", "LTLT照明灯具","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELTCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.7-照明系统设备.xlsx", "LTCU照明控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELTCP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.7-照明系统设备.xlsx", "LTCP照明控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELTCC", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.7-照明系统设备.xlsx", "LTCC照明回路（预计330后删除）","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELTLG", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.7-照明系统设备.xlsx", "LTLG照明灯组（预计330后删除）","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETPCU", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.8-租户配电系统设备.xlsx", "TPCU租户配电控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETPBP", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.8-租户配电系统设备.xlsx", "TPBP租户母线插接箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELPAT", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.9-雷电防护系统设备.xlsx", "LPAT接闪器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELPDL", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.9-雷电防护系统设备.xlsx", "LPDL引下线","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SELPGD", "/附表8-设备类对象分类属性点表/附表8.1-强电专业设备/附表8.1.9-雷电防护系统设备.xlsx", "LPGD接地装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBAHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BAHT楼控主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBACU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BACU楼控控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBADC", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BADC楼控数据采集器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBAPS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BAPS楼控电源供应器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBAEC", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BAEC楼控机柜","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBADS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BADS楼控显示屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBACP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.1-建筑设备控制系统设备.xlsx", "BACP楼控控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEEMHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.10-能源管理系统设备.xlsx", "EMHT能源管理主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEEMCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.10-能源管理系统设备.xlsx", "EMCU能源管理控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMMHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.11-气象监测系统设备.xlsx", "MMHT气象监测主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMMCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.11-气象监测系统设备.xlsx", "MMCU气象监测控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMMMS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.11-气象监测系统设备.xlsx", "MMMS气象监测站","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WECMHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.12-收银管理系统设备.xlsx", "CMHT收银管理主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WECMCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.12-收银管理系统设备.xlsx", "CMCU收银管理控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFTL", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFTL办公电话","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFCM", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFCM办公摄像头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFCP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFCP办公控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFCU办公控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFDS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFDS办公显示屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFFE", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFFE办公前端设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFHT办公主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFSP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFSP办公扬声器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFCK", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFCK办公时钟","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFPT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFPT办公打印机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFMP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFMP办公话筒","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFPJ", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFPJ办公投影仪","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFPS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFPS办公投影幕","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEOFTV", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.13-办公系统设备.xlsx", "OFTV办公电视机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETVFE", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.2-电视信号系统设备.xlsx", "TVFE电视前端设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETVTV", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.2-电视信号系统设备.xlsx", "TVTV电视机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETVCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.2-电视信号系统设备.xlsx", "TVCU电视信号控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMIHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.3-多媒体信息发布系统设备.xlsx", "MIHT多媒体主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMIPL", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.3-多媒体信息发布系统设备.xlsx", "MIPL播放器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMIDS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.3-多媒体信息发布系统设备.xlsx", "MIDS多媒体显示屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMIST", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.3-多媒体信息发布系统设备.xlsx", "MIST自助终端机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMIFE", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.3-多媒体信息发布系统设备.xlsx", "MIFE多媒体前端设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMICU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.3-多媒体信息发布系统设备.xlsx", "MICU多媒体控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEWTWT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.4-无线对讲系统设备.xlsx", "WTWT对讲机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEWTRS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.4-无线对讲系统设备.xlsx", "WTRS中继台","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEWTCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.4-无线对讲系统设备.xlsx", "WTCU无线对讲控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMHT车库主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMCU车库控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMCM", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMCM车库摄像头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMDS", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMDS车库显示屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMGT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMGT进出闸机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMMP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMMP机械车位","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMCP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMCP充电桩","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMPD", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMPD车位检测器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMGL", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMGL车位引导灯","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMLP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMLP升降防撞柱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEGMPL", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.5-车库管理系统设备.xlsx", "GMPL车库地锁","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNHT服务器主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNRT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNRT路由器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNEX", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNEX交换机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNHF", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNHF硬件防火墙","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNAC", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNAC无线控制器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNAP", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNAP无线AP点位","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WELNCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.6-本地网络系统设备.xlsx", "LNCU网络控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMAHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.7-移动通信信号放大系统设备.xlsx", "MAHT移动通信信号放大主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMAAT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.7-移动通信信号放大系统设备.xlsx", "MAAT移动通信信号放大天线","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEMACU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.7-移动通信信号放大系统设备.xlsx", "MACU移动通信信号放大控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETMHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.8-租户管理系统设备.xlsx", "TMHT租户管理主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WETMCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.8-租户管理系统设备.xlsx", "TMCU租户管理控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEPMHT", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.9-客流管理系统设备.xlsx", "PMHT客流管理主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEPMCU", "/附表8-设备类对象分类属性点表/附表8.2-弱电专业设备/附表8.2.9-客流管理系统设备.xlsx", "PMCU客流管理控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCCC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCCC离心式冷水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCSC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCSC螺杆式冷水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCPC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCPC活塞式冷水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCAC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCAC吸收式冷水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCAH", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCAH空调空气源热泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCWH", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCWH空调水源热泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCGH", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCGH空调地源热泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCOT", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCOT冷却塔","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCCP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCCP供冷冷冻水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCOP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCOP供冷冷却水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCGP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCGP供冷乙二醇泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCFP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCFP供冷补水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCCF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCCF供冷定压补水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCPT", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCPT供冷压力罐","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCHE", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCHE供冷换热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCVD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCVD供冷真空脱气机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCCD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCCD供冷水加药装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCSD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCSD供冷软化水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCTU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCTU供冷全程水处理仪","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCST", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCST供冷蓄冷热槽","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCDB", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCDB供冷分水器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCCL", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCCL供冷集水器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCWT", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCWT供冷水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCOC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCOC供冷在线清洗装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCTC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCTC供冷伴热电缆","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCCU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.1-中央供冷系统设备.xlsx", "CCCU供冷控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHCB", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHCB供热燃煤锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHFB", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHFB供热燃油锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHGB", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHGB供热燃气锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHEB", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHEB供热电锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHHP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHHP供热水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHFP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHFP供热补水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHCF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHCF供热定压补水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHPT", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHPT供热压力罐","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHHE", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHHE供热换热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHVD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHVD供热真空脱气机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHCD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHCD供热水加药装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHSD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHSD供热软化水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHTU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHTU供热全程水处理仪","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHST", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHST供热蓄冷热槽","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHDB", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHDB供热分水器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHCL", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHCL供热集水器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHWT", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHWT供热水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHOC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHOC供热在线清洗装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHBF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHBF生物质炊暖炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCHCU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.2-中央供热系统设备.xlsx", "CHCU供热控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATAH", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATAH空调机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATFU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATFU新风机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATFC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATFC风机盘管","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATSA", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATSA分体空调","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATSI", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATSI分体空调室内机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATSO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATSO分体空调室外机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATVR", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATVR变频多联机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATVI", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATVI变频多联机室内机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATVO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATVO变频多联机室外机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATVA", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATVA变风量箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATFA", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATFA地板空调器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATIO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATIO空调风口","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATRD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATRD散热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATIR", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATIR红外辐射单元","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATFH", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATFH地板采暖设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATHD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATHD空调加湿器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATMF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATMF空调消声器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATPC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATPC空调静压箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATCA", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATCA定风量箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATCP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATCP空调末端控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACATCU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.3-空调末端系统设备.xlsx", "ATCU空调末端控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTSF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTSF通风风机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTAC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTAC热风幕","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTAP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTAP空气净化器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTDO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTDO通风除味装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTIO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTIO通风风口","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTMF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTMF通风消声器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTPC", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTPC通风静压箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTCU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTCU通风控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACVTCP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.4-通风系统设备.xlsx", "VTCP通风控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACKLSF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.5-厨房排油烟系统设备.xlsx", "KLSF厨房排油烟风机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACKLAP", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.5-厨房排油烟系统设备.xlsx", "KLAP厨房油烟净化器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACKLCU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.5-厨房排油烟系统设备.xlsx", "KLCU厨房排油烟控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACKLIO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.5-厨房排油烟系统设备.xlsx", "KLIO厨房排油烟风口","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACADSF", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.6-空调人防系统设备.xlsx", "ADSF空调人防风机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACADFD", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.6-空调人防系统设备.xlsx", "ADFD滤毒除湿机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACADCU", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.6-空调人防系统设备.xlsx", "ADCU空调人防控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACADIO", "/附表8-设备类对象分类属性点表/附表8.3-空调专业设备/附表8.3.6-空调人防系统设备.xlsx", "ADIO空调人防风口","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWSU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWSU生活给水供水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWTS", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWTS生活给水水箱消毒器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWUS", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWUS生活给水紫外线消毒器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWWT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWWT生活给水储水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWPT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWPT生活给水压力罐","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWPP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWPP生活给水水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDWCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.1-生活给水系统设备.xlsx", "DWCU生活给水控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWHTC", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.10-给排水电伴热系统设备.xlsx", "WHTC给排水伴热电缆","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWHIB", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.10-给排水电伴热系统设备.xlsx", "WHIB给排水伴热保温器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWHCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.10-给排水电伴热系统设备.xlsx", "WHCU给排水伴热控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWDWT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.11-给排水人防系统设备.xlsx", "WDWT给排水人防储水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWDCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.11-给排水人防系统设备.xlsx", "WDCU给排水人防控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHCB", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHCB生活热水燃煤锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHFB", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHFB生活热水燃油锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHGB", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHGB生活热水燃气锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHEB", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHEB生活热水电锅炉","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHAH", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHAH生活热水空气源热泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHGH", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHGH生活热水燃气热水器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHEH", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHEH生活热水电热水器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHSC", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHSC生活热水太阳能集热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHHA", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHHA生活热水辅热设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHHE", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHHE生活热水换热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WS DHS", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", " DHSU生活热水供水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHHP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHHP生活热水水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHSD", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHSD生活热水软化水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHCF", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHCF生活热水定压补水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHPT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHPT生活热水压力罐","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHST", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHST生活热水蓄冷热槽","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHWT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHWT生活热水储水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDHCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.2-生活热水系统设备.xlsx", "DHCU生活热水控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWSU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWSU中水供水机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWTU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWTU中水处理机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWWT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWWT中水储水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWRC", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWRC雨水收集设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWPT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWPT中水供水压力罐","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWPP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWPP中水供水水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWCD", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWCD中水水加药装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSLAFT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "LAFT中水过滤器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSRWCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.3-中水系统设备.xlsx", "RWCU中水控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDDTU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.4-直饮水系统设备.xlsx", "DDTU直饮水处理机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDDIH", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.4-直饮水系统设备.xlsx", "DDIH直饮水即时加热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSDDCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.4-直饮水系统设备.xlsx", "DDCU直饮水控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTLU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STLU污废水一体式提升机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTSP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STSP潜污泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTOR", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STOR餐饮除油成套设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTST", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STST化粪池","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTCP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STCP集水坑","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTOS", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STOS隔油池","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTQM", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STQM污水水质监测仪","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSTCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.5-污水处理系统设备.xlsx", "STCU污水控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSISU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.6-喷灌系统设备.xlsx", "SISU喷灌机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSISN", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.6-喷灌系统设备.xlsx", "SISN喷灌喷头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSICU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.6-喷灌系统设备.xlsx", "SICU喷灌控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWSCF", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.7-水景系统设备.xlsx", "WSCF水景循环过滤机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWSSN", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.7-水景系统设备.xlsx", "WSSN水景喷头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWSCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.7-水景系统设备.xlsx", "WSCU水景控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWSLT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.7-水景系统设备.xlsx", "WSLT水景灯具","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWSPP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.7-水景系统设备.xlsx", "WSPP水景水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPCF", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPCF泳池循环过滤机组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPPP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPPP泳池循环水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPHE", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPHE泳池换热器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPOS", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPOS泳池臭氧消毒器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPQM", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPQM泳池水质监测仪","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPPB", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPPB泳池PH值平衡投药泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPOP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPOP泳池氧消毒剂投药泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPWT", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPWT泳池水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSWPCU", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.8-泳池系统设备.xlsx", "WPCU泳池控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSFGPA", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.9-燃气系统设备.xlsx", "FGPA燃气调压箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSFGLP", "/附表8-设备类对象分类属性点表/附表8.4-给排水专业设备/附表8.4.9-燃气系统设备.xlsx", "FGLP液化石油气瓶","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFAHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FAHT火灾报警主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFADS", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FADS火灾报警显示屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFALP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FALP联动琴台","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFASE", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FASE火灾探测器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFAAL", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FAAL火灾报警声光报警器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFAMA", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FAMA消防手动报警按钮","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFAHA", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FAHA消火栓报警按钮","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFATL", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FATL消防电话","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFACU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.1-火灾报警系统设备.xlsx", "FACU火灾报警控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEEHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.10-应急照明及疏散指示系统设备.xlsx", "EEHT应急照明及疏散指示主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEECU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.10-应急照明及疏散指示系统设备.xlsx", "EECU应急照明及疏散指示控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEEEL", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.10-应急照明及疏散指示系统设备.xlsx", "EEEL应急照明灯具","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEEEI", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.10-应急照明及疏散指示系统设备.xlsx", "EEEI疏散指示灯牌","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFKECU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.11-厨房自动灭火系统设备.xlsx", "KECU厨房自动灭火控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFKEEC", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.11-厨房自动灭火系统设备.xlsx", "KEEC厨房自动灭火灭火剂容器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFKESN", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.11-厨房自动灭火系统设备.xlsx", "KESN厨房自动灭火喷头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGAHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.2-燃气报警系统设备.xlsx", "GAHT燃气报警主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGASE", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.2-燃气报警系统设备.xlsx", "GASE燃气探测器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGACV", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.2-燃气报警系统设备.xlsx", "GACV燃气切断阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGACU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.2-燃气报警系统设备.xlsx", "GACU燃气报警控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEAHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.3-漏电火灾报警系统设备.xlsx", "EAHT漏电火灾报警主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEALS", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.3-漏电火灾报警系统设备.xlsx", "EALS漏电探测器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFEACU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.3-漏电火灾报警系统设备.xlsx", "EACU漏电火灾报警控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBMFE", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.4-紧急广播及背景音乐系统设备.xlsx", "BMFE广播前端设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBMHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.4-紧急广播及背景音乐系统设备.xlsx", "BMHT广播主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBMCU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.4-紧急广播及背景音乐系统设备.xlsx", "BMCU广播分区控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBMDS", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.4-紧急广播及背景音乐系统设备.xlsx", "BMDS广播显示屏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBMMP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.4-紧急广播及背景音乐系统设备.xlsx", "BMMP广播话筒","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFBMSP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.4-紧急广播及背景音乐系统设备.xlsx", "BMSP广播扬声器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSHP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSHP消火栓供水加压泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSSP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSSP喷淋供水加压泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSMP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSMP水喷雾供水加压泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSAP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSAP停机坪消防供水加压泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSPP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSPP预作用加压泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSGP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSGP窗玻璃喷淋泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSIP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSIP加密喷淋泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSWP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSWP水炮泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSPM", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSPM消防稳压泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSWC", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSWC消防水幕泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSWM", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSWM消防水炮设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSCF", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSCF消防给水定压补水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSPT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSPT消防给水压力罐","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSFT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSFT泡沫储罐组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSAC", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSAC消防空压机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSWV", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSWV湿式报警阀组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSDV", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSDV干式报警阀组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSPV", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSPV预作用报警阀组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSHB", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSHB消火栓箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSOH", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSOH室外消火栓","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSPC", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSPC消防水泵接合器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSTC", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSTC消防伴热电缆","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSET", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSET消防末端试水装置","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSWT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSWT消防水箱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSTS", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSTS消防水箱消毒器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSFP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSFP消防补水泵","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSSN", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSSN消防喷头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSCP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSCP消火栓控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSHT消防给水主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFSCU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.5-消防给水系统设备.xlsx", "FSCU消防给水控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGEBG", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.6-气体灭火系统设备.xlsx", "GEBG气体灭火瓶组","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGESB", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.6-气体灭火系统设备.xlsx", "GESB气体灭火启动瓶","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGEHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.6-气体灭火系统设备.xlsx", "GEHT气体灭火主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGECU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.6-气体灭火系统设备.xlsx", "GECU气体灭火控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFGESN", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.6-气体灭火系统设备.xlsx", "GESN气体灭火喷头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFEFE", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.7-灭火器系统设备.xlsx", "FEFE灭火器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSCEF", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.8-防排烟系统设备.xlsx", "SCEF防排烟风机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSCIO", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.8-防排烟系统设备.xlsx", "SCIO排烟风口","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSCHW", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.8-防排烟系统设备.xlsx", "SCHW电动挡烟垂壁","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSCSD", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.8-防排烟系统设备.xlsx", "SCSD防火排烟阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSCCU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.8-防排烟系统设备.xlsx", "SCCU防排烟控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFRFR", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.9-防火卷帘门系统设备.xlsx", "FRFR防火卷帘门","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFRFD", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.9-防火卷帘门系统设备.xlsx", "FRFD防火门","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFRHT", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.9-防火卷帘门系统设备.xlsx", "FRHT防火卷帘门主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFRCP", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.9-防火卷帘门系统设备.xlsx", "FRCP防火卷帘门控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFFRCU", "/附表8-设备类对象分类属性点表/附表8.5-消防专业设备/附表8.5.9-防火卷帘门系统设备.xlsx", "FRCU防火卷帘门控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSHT", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSHT监控主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSCM", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSCM监控摄像头","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSFE", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSFE监控前端设备","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSVM", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSVM监视器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSDR", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSDR硬盘录像机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSCP", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSCP监控控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPVSCU", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.1-视频监控系统设备.xlsx", "VSCU监控控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPNPHT", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.2-巡更系统设备.xlsx", "NPHT巡更主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPNPCU", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.2-巡更系统设备.xlsx", "NPCU巡更控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPNPCP", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.2-巡更系统设备.xlsx", "NPCP巡更通讯座","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPNPPS", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.2-巡更系统设备.xlsx", "NPPS巡更棒","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGHT", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGHT门禁主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGCU", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGCU门禁控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGCR", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGCR门禁读卡器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGWT", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGWT门禁对讲机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGTK", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGTK破胎器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGDL", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGDL门禁门锁","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGBG", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGBG破玻璃按钮","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGCP", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGCP门禁控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPEGFG", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.3-门禁系统设备.xlsx", "EGFG速通门","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIAHT", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.4-入侵报警系统设备.xlsx", "IAHT入侵报警主机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIACU", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.4-入侵报警系统设备.xlsx", "IACU入侵报警控制模块","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIAMA", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.4-入侵报警系统设备.xlsx", "IAMA入侵报警手动报警按钮","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIAAL", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.4-入侵报警系统设备.xlsx", "IAAL入侵报警声光报警器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIACP", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.4-入侵报警系统设备.xlsx", "IACP入侵报警控制面板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPIASE", "/附表8-设备类对象分类属性点表/附表8.6-安防专业设备/附表8.6.4-入侵报警系统设备.xlsx", "IASE入侵报警探测器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFD", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSFD基础","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSCL", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSCL柱","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSGD", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSGD梁","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSTR", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSTR桁架","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSWL", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSWL墙","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFL", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSFL楼板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSRF", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSRF屋顶","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSSW", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.1-土建结构设备.xlsx", "CSSW楼梯","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBECW", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BECW幕墙","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBEBC", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BEBC呼吸式幕墙","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBELR", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BELR采光顶","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBEDR", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BEDR门","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBEWN", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BEWN窗","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBEAW", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BEAW雨篷","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBEES", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BEES电动遮阳帘","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBEMT", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BEMT遮阳电机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBECT", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.2-围护结构设备.xlsx", "BECT普通遮阳帘","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDWS", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDWS盥洗盆","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDTL", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDTL坐便器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDSP", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDSP蹲便器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDUR", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDUR小便器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDHD", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDHD干手机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDBT", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDBT浴缸","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDSK", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDSK洗碗槽","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDSC", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDSC插座","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDFN", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDFN家具","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDCL", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDCL吊顶","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFIDFD", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.3-室内装饰设备.xlsx", "IDFD地漏","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCADM", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CADM装饰材料","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAHD", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAHD栏杆","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCASB", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CASB减速带","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAST", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAST地桩","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCADB", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CADB展示板","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAID", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAID标识","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAWC", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAWC擦窗机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCADD", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CADD排水沟","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAWB", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAWB垃圾桶","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAMC", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAMC井盖","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAIS", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAIS检查井","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCAOP", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.4-建筑附件设备.xlsx", "CAOP室外道路","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFPKPS", "/附表8-设备类对象分类属性点表/附表8.7-土建精装专业设备/附表8.7.5-停车场设备.xlsx", "PKPS停车位","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSAEW", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.1-强电线路及附件设备.xlsx", "SAEW强电线路","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTWAEW", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.2-弱电线路及附件设备.xlsx", "WAEW弱电线路","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTLAPL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.3-液体管道及附件设备.xlsx", "LAPL液体管道","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTLAFI", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.3-液体管道及附件设备.xlsx", "LAFI水流指示器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTLAFT", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.3-液体管道及附件设备.xlsx", "LAFT水管过滤器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAAPL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.4-气体管道及附件设备.xlsx", "AAPL气体管道","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAAFT", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.4-气体管道及附件设备.xlsx", "AAFT风道过滤器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSECB", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SECB组合式传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSETP", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SETP温度传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSERH", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SERH湿度传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEPS", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEPS压力传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSECT", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SECT浓度传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSESP", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SESP速度传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEDP", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEDP位移传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEFL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEFL流量传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEHT", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEHT热量传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEEE", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEEE电度传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSESD", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SESD声音传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEVD", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEVD图像传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEPF", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEPF人流量传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEIL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEIL照度传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSEIP", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.5-传感器设备.xlsx", "SEIP室内定位传感器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAEOV", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AEOV通断阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAEAV", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AEAV调节阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAECV", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AECV止回阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAERV", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AERV减压阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAESV", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AESV安全阀","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAERL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AERL继电器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAEFC", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AEFC变频器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAEIL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AEIL提示灯","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTAEAL", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.6-执行器设备.xlsx", "AEAL声光报警器","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTXXFZ", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.7-其他设备.xlsx", "XXFZ冷柜（恒温柜）","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTXXDC", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.7-其他设备.xlsx", "XXDC风干柜","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTXXAC", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.7-其他设备.xlsx", "XXAC冷风机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTXXSC", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.7-其他设备.xlsx", "XXSC消毒柜","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTXXUA", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.7-其他设备.xlsx", "XXUA无人机","2",equipCommonSeq);
        equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTXXCR", "/附表8-设备类对象分类属性点表/附表8.8-其他专业设备/附表8.8.7-其他设备.xlsx", "XXCR扫地机器人","2",equipCommonSeq);

    }

    private void importComponent(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("component", "/附表6-机电精装类对象通用属性点表.xlsx", "部件类","1");
        AtomicInteger componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDHSIP", "/附表9-部件类对象分类属性点表/附表9.1-强电专业部件.xlsx", "TDHSIP综合保护装置","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SETDLSDW", "/附表9-部件类对象分类属性点表/附表9.1-强电专业部件.xlsx", "TDLSDW低压配电抽屉","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WEBADCBP", "/附表9-部件类对象分类属性点表/附表9.2-弱电专业部件.xlsx", "BADCBP总线端口","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCSCMT", "/附表9-部件类对象分类属性点表/附表9.3-空调专业部件.xlsx", "CCSCMT螺杆式冷水机组电机","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCGHBP", "/附表9-部件类对象分类属性点表/附表9.3-空调专业部件.xlsx", "CCGHBP地埋管","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("ACCCOTFN", "/附表9-部件类对象分类属性点表/附表9.3-空调专业部件.xlsx", "CCOTFN冷却塔风机","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("WSSheet1", "/附表9-部件类对象分类属性点表/附表9.4-给排水专业部件.xlsx", "Sheet1","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("FFSheet1", "/附表9-部件类对象分类属性点表/附表9.5-消防专业部件.xlsx", "Sheet1","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("SPSheet1", "/附表9-部件类对象分类属性点表/附表9.6-安防专业部件.xlsx", "Sheet1","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDCS", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDCS基础垫层","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDWB", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDWB基础防水板","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDGD", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDGD基础梁","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDPN", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDPN基础嵌板","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDAA", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDAA基础抗浮锚杆","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDPL", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDPL基础桩","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDED", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDED基础电梯基坑","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSFDCP", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSFDCP基础集水坑","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSWLBD", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSWLBD墙体","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSWLIL", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSWLIL墙保温层","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSWLEP", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSWLEP墙预埋件","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSWLCT", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSWLCT墙涂层","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSSWBD", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSSWBD楼梯主体","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSSWRW", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSSWRW楼梯坡道","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFCSSWAR", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "CSSWAR楼梯扶手","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBECWKL", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "BECWKL幕墙龙骨","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBECWPN", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "BECWPN幕墙嵌板","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBECWCP", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "BECWCP幕墙组件","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBELRKL", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "BELRKL采光顶龙骨","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBELRPN", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "BELRPN采光顶嵌板","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("CFBELRCP", "/附表9-部件类对象分类属性点表/附表9.7-土建精装专业部件.xlsx", "BELRCP采光顶组件","2",componentCommonSeq);
        componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());
        analysisSheetDataAndWriteToFile("OTSheet1", "/附表9-部件类对象分类属性点表/附表9.8-其他专业部件.xlsx", "Sheet1","2",componentCommonSeq);
        //componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());

    }

    private void importVirtual(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("VOEm", "/附表13-虚拟对象属性点表.xlsx", "能耗模型","2");
        analysisSheetDataAndWriteToFile("VOSe", "/附表13-虚拟对象属性点表.xlsx", "分项","2");
        analysisSheetDataAndWriteToFile("VOBc", "/附表13-虚拟对象属性点表.xlsx", "信标","2");
        analysisSheetDataAndWriteToFile("VOTn", "/附表13-虚拟对象属性点表.xlsx", "租户","2");
        analysisSheetDataAndWriteToFile("VoWs", "/附表13-虚拟对象属性点表.xlsx", "工作历","2");
        analysisSheetDataAndWriteToFile("VoBl", "/附表13-虚拟对象属性点表.xlsx", "集团","2");
    }

    private void importTool(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("tool", "/附表10-耗材工具类对象通用属性点表.xlsx", "工具类","2");
    }

    private void importMaterial(List<FuncidDefModel> funcids) throws IOException {
        analysisSheetDataAndWriteToFile("material", "/附表10-耗材工具类对象通用属性点表.xlsx", "耗材类","2");
    }

    @Data
    private static class CacheModel {
        private String classCode;
        private String firstTag;
        private String secondTag;
        private String origCode;
        private String name;
        private String unit;
        private String origDataType;
        private String priority; // 优先级
        private String inputMode;
        private String origDataSource; // 字典选择
        private String remark; // 备注
        private String shareType; // 通用信息点标记,1-通用信息点,2-个性化信息点
        private Integer orderSeq; // 排序序号
    }

    private List<FuncidDefModel> readSheet(String classCode, String fileName, String sheetName) throws IOException {
        return readSheet(classCode,fileName,sheetName,"2");
    }

    private List<FuncidDefModel> readSheet(String classCode, String fileName, String sheetName,String shareType) throws IOException {
        return readSheet(classCode,fileName,sheetName,shareType,new AtomicInteger(0));
    }

    private List<FuncidDefModel> readSheet(String classCode, String fileName, String sheetName,String shareType,AtomicInteger commonSeq) throws IOException {
        List<FuncidDefModel> data = new LinkedList<>();
        String classFile = basepath + fileName;
        FileInputStream inputStream = new FileInputStream(classFile);
        Workbook workbook = new XSSFWorkbook(inputStream);
        Sheet sheet = workbook.getSheet(sheetName);
        if (sheet == null) {
            return data;
        }
        // 行号从0开始, 从第2行开始读数据
        int startRowNum = 2;
        int lastRowNum = sheet.getLastRowNum();
        CacheModel cache = new CacheModel();
        for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
            Row row = sheet.getRow(rowIndex);
            if (row == null) {
                log.info("file: {}, sheet: {}, row[{}] is null", fileName, sheetName, rowIndex);
                continue;
            }
            // 0:firstTag, 1:secondTag, 2:name, 3:origCode, 4:unit, 5:origDataType, 6:priorite, 7:imputMode, 8:origDataSource, 9:remark
            CacheModel target = new CacheModel();
            target.classCode = classCode;
            target.firstTag = (String) ExcelUtils.parseCell(row.getCell(0), "string");
            target.secondTag = (String) ExcelUtils.parseCell(row.getCell(1), "string");
            target.name = (String) ExcelUtils.parseCell(row.getCell(2), "string");
            target.origCode = (String) ExcelUtils.parseCell(row.getCell(3), "string");
            target.unit = (String) ExcelUtils.parseCell(row.getCell(4), "string");
            target.origDataType = (String) ExcelUtils.parseCell(row.getCell(5), "string");
            target.priority = (String) ExcelUtils.parseCell(row.getCell(6), "string");
            target.inputMode = (String) ExcelUtils.parseCell(row.getCell(7), "string");
            target.origDataSource = (String) ExcelUtils.parseCell(row.getCell(8), "string");
            target.remark = (String) ExcelUtils.parseCell(row.getCell(9), "string");
            target.shareType = shareType;
            target.orderSeq = commonSeq.addAndGet(1);
            if (target.origCode == null || "".equals(target.origCode) || "`".equals(target.origCode)) {
                continue;
            }
            // firstTag不为空
            if (target.firstTag != null) {
                cache.firstTag = target.firstTag;
                cache.secondTag = null;
            } else {
                target.firstTag = cache.firstTag;
            }

            // secondTag
            if (target.secondTag != null) {
                cache.secondTag = target.secondTag;
            } else {
                target.secondTag = cache.secondTag;
            }
            FuncidDefModel funcidDefModel = toFuncidDefModel(target);
            if (funcidDefModel == null) {
                log.info("file: {}, sheet: {}, row: {} data type [{}] error", fileName, sheetName, rowIndex, target.origDataType);
            } else {
                if (FuncidUtils.isBase(target.origCode)) {
                    funcidDefModel.setCode(FuncidUtils.getRealCode(target.origCode));
                }
                data.add(funcidDefModel);
            }
        }
        if ("1".equals(shareType)){
            //通用的信息点,则重置一下排序
            int seq = -1;
            for (int i = data.size() - 1; i >= 0; i--) {
                FuncidDefModel funcidDefModel = data.get(i);
                funcidDefModel.setOrderSeq(seq--);
            }
        }
        return data;
    }

    @Test
    public void test() {
        ArrayNode array = JsonNodeFactory.instance.arrayNode();
        //查询知识库详情
        String url = rwdeditServer + "/rwdedit/ref/type/infos/query";
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("refKey", "classificationSystem");
        HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 300000);
        String content = response.getContent();
        ObjectNode objectNode = JacksonMapper.toObject(content, ObjectNode.class);
        ArrayNode data = JacksonMapper.getArray(objectNode, "data", JsonNodeFactory.instance.arrayNode());
        for (JsonNode datum : data) {
            ObjectNode obj = (ObjectNode) datum;
            ObjectNode item = array.addObject();
            item.put("code", JacksonMapper.getString(obj, "code"));
            item.put("name", JacksonMapper.getString(obj, "name"));
            String parentCode = JacksonMapper.getString(obj, "parentCode");
            if (parentCode != null) {
                item.put("parentCode", parentCode);
            }
        }
        System.err.println(JacksonMapper.toSimpleJson(array));
    }

}

